@@ -20,9 +20,10 @@ from collections import defaultdict
 from nose.plugins.base import Plugin
 from subprocess import Popen, PIPE, STDOUT
 
-from kleenex.db import TestCoverageDB
+from kleenex.db import CoverageDB
 from kleenex.diff import DiffParser
 from kleenex.tracer import ExtendedTracer
+from kleenex.config import read_config
 
 def is_py_script(filename):
     "Returns True if a file is a python executable."
@@ -85,68 +86,15 @@ class TestCoveragePlugin(Plugin):
 
     def options(self, parser, env):
         Plugin.options(self, parser, env)
-        parser.add_option('--kleenex-parent',
-                          dest="coverage_parent",
-                          default="origin/master")
-
-        parser.add_option("--kleenex-record",
-                          dest="record_test_coverage", action="store_true",
-                          default=False)
-
-        parser.add_option("--no-kleenex-record",
-                          dest="record_test_coverage", action="store_false",
-                          default=False)
-
-        parser.add_option("--kleenex-report",
-                          dest="report_test_coverage", action="store_true",
-                          default=False)
-
-        parser.add_option("--no-kleenex-report",
-                          dest="report_test_coverage", action="store_false",
-                          default=False)
-
-        parser.add_option("--kleenex-skip-missing",
-                          dest="skip_missing_coverage", action="store_true",
-                          default=None)
-
-        parser.add_option("--no-kleenex-skip-missing",
-                          dest="skip_missing_coverage", action="store_false",
-                          default=None)
-
-        parser.add_option("--kleenex-discover",
-                          dest="discover", action="store_true",
-                          default=True)
-
-        parser.add_option("--no-kleenex-discover",
-                          dest="discover", action="store_false",
-                          default=True)
-
-        parser.add_option("--kleenex-dsn",
-                          dest="coverage_dsn",
-                          default='sqlite:///coverage.db')
-
-        parser.add_option('--with-kleenex-file', action='store_true',
-                          dest="with_coverage_file",
-                          default=False)
-
-        parser.add_option('--kleenex-file', action='store',
-                          dest='coverage_file', metavar="FILE",
-                          default=env.get('NOSE_BLEED_FILE', 'kleenex.json'),
-                          help=("Path to json file to store the report in. "
-                                "Default is kleenex.json in the working directory "
-                                "[NOSE_BLEED_FILE]"))
+        parser.add_option("--kleenex-config", dest="kleenex_config", default="setup.cfg")
 
     def configure(self, options, config):
         Plugin.configure(self, options, config)
-        self.skip_missing = options.skip_missing_coverage
-        self.record = options.record_test_coverage
-        self.report_coverage = options.report_test_coverage
-        self.discover = options.discover
+        config = read_config(options.kleenex_config)
+
+        self.config = config
+
         self.logger = logging.getLogger(__name__)
-        self.dsn = options.coverage_dsn
-        self.parent = options.coverage_parent
-        self.coverage_max_proximity = 4
-        # self.enabled = (self.record or self.report_coverage or self.discover)
 
         if not self.enabled:
             return
@@ -157,38 +105,43 @@ class TestCoveragePlugin(Plugin):
         # cov is a mapping of filename->[linenos]
         self.cov_data = defaultdict(set)
 
-        if options.with_coverage_file:
-            self.report_file = open(options.coverage_file, 'w')
-        else:
+        report_output = config.report_output
+        if not report_output:
             self.report_file = None
+        elif report_output.startswith('sys://'):
+            pipe = report_output[6:]
+            assert pipe in ('stdout', 'stderr')
+            self.report_file = getattr(sys, pipe)
+        else:
+            self.report_file = open(report_output, 'w')
 
     def begin(self):
         # XXX: this is pretty hacky
-        self.db = TestCoverageDB(self.dsn, self.logger)
-        if self.record:
+        self.db = CoverageDB(self.config.db, self.logger)
+        if self.config.record:
             self.db.upgrade()
 
         self.revision = _get_git_revision('.git')
 
-        if self.report_coverage or self.record:
+        if self.config.report or self.config.record:
             # If we're recording coverage we need to ensure it gets reset
             self.coverage = self._setup_coverage()
 
-        if not (self.discover or self.report_coverage):
+        if not (self.config.discover or self.config.report):
             return
 
         s = time.time()
         self.logger.info("Getting current revision")
         # pull in our diff
         # git diff `git merge-base HEAD master`
-        proc = Popen(['git', 'diff', self.parent], stdout=PIPE, stderr=STDOUT)
+        proc = Popen(['git', 'diff', self.config.parent], stdout=PIPE, stderr=STDOUT)
         diff = proc.stdout.read()
 
         s = time.time()
-        self.logger.info("Parsing diff from parent %s", self.parent)
+        self.logger.info("Parsing diff from parent %s", self.config.parent)
         # pull in our diff
         # git diff `git merge-base HEAD master`
-        proc = Popen(['git', 'diff', self.parent], stdout=PIPE, stderr=STDOUT)
+        proc = Popen(['git', 'diff', self.config.parent], stdout=PIPE, stderr=STDOUT)
         diff = proc.stdout.read()
 
         pending_funcs = self.pending_funcs
@@ -215,7 +168,7 @@ class TestCoveragePlugin(Plugin):
             else:
                 filename = file['old_filename']
                 if not filename.startswith('a/'):
-                    continue # ??
+                    continue  # ??
 
             filename = filename[2:]
 
@@ -233,12 +186,12 @@ class TestCoveragePlugin(Plugin):
             if is_new_file:
                 continue
 
-            if not self.discover:
+            if not self.config.discover:
                 continue
 
         self.logger.info("Parsed diff in %.2fs as %d file(s)", time.time() - s, len(diff))
 
-        if self.discover:
+        if self.config.discover:
             self.logger.info("Finding coverage for %d file(s)", len(files))
 
             for filename, linenos in diff.iteritems():
@@ -246,7 +199,7 @@ class TestCoveragePlugin(Plugin):
                 if not test_coverage:
                     # check if we have any coverage recorded
                     if not self.db.has_test_coverage(filename):
-                        if self.skip_missing:
+                        if self.config.skip_missing:
                             self.logger.warning('%s has no test coverage recorded', filename)
                             continue
                         raise AssertionError("Missing test coverage for %s" % filename)
@@ -257,7 +210,7 @@ class TestCoveragePlugin(Plugin):
             self.logger.info("Determined available coverage in %.2fs with %d test(s)", time.time() - s, len(pending_funcs))
 
     def report(self, stream):
-        if not self.report_coverage:
+        if not self.config.report:
             return
 
         cov_data = self.cov_data
@@ -285,20 +238,19 @@ class TestCoveragePlugin(Plugin):
             self.report_file.close()
         elif total:
             stream.writeln('Coverage Report')
-            stream.writeln('-'*70)
+            stream.writeln('-' * 70)
             stream.writeln('Coverage against diff is %.2f%% (%d / %d lines)' % (covered / float(total) * 100, covered, total))
             if missing:
                 stream.writeln()
                 stream.writeln('%-35s   %s' % ('Filename', 'Missing Lines'))
-                stream.writeln('-'*70)
+                stream.writeln('-' * 70)
                 for filename, linenos in missing.iteritems():
                     if not linenos:
                         continue
                     stream.writeln('%-35s   %s' % (filename, ', '.join(map(str, sorted(linenos)))))
 
-
     def wantMethod(self, method):
-        if not self.discover:
+        if not self.config.discover:
             return
 
         # only works with unittest compatible functions currently
@@ -311,7 +263,7 @@ class TestCoveragePlugin(Plugin):
             return True
 
         # test has no coverage recorded, defer to other plugins
-        elif not self.db.has_seen_test(test_name):
+        elif self.config.test_missing and not self.db.has_seen_test(test_name):
             self.pending_funcs.add(test_name)
             self.logger.info("Allowing test due to missing coverage report: %s", test_name)
             return None
@@ -319,13 +271,13 @@ class TestCoveragePlugin(Plugin):
         return False
 
     def startTest(self, test):
-        if not (self.record or self.report_coverage):
+        if not (self.config.record or self.config.report):
             return
 
         self.coverage.start()
 
     def stopTest(self, test):
-        if not (self.record or self.report_coverage):
+        if not (self.config.record or self.config.report):
             return
 
         cov = self.coverage
@@ -348,9 +300,9 @@ class TestCoveragePlugin(Plugin):
         # The rest of this is all run within a single transaction
         trans = self.db.begin()
 
-        if self.record:
+        if self.config.record:
             self.db.clear_test_coverage(test_name)
-            self.db.set_test_has_coverage(test_name, self.revision)
+            self.db.set_test_has_seen_test(test_name, self.revision)
 
         # Compute the standard deviation for all code executed from this test
         linenos = []
@@ -362,10 +314,11 @@ class TestCoveragePlugin(Plugin):
             #     continue
             filename = cu.name + '.py'
             linenos = cov.data.executed_lines(cu.filename)
-            linenos_in_prox = dict((k, v) for k, v in linenos.iteritems() if v < self.coverage_max_proximity)
-            if self.record and linenos_in_prox:
-                self.db.set_test_coverage(test_name, filename, linenos_in_prox)
-            if self.report_coverage:
+            if self.config.record:
+                linenos_in_prox = dict((k, v) for k, v in linenos.iteritems() if v < self.config.max_distance)
+                if linenos_in_prox:
+                    self.db.set_test_coverage(test_name, filename, linenos_in_prox)
+            if self.config.report:
                 diff = self.diff_data[filename]
                 cov_linenos = [l for l in linenos if l in diff]
                 if cov_linenos:
